/*
 *  Copyright (C) 2004 Link Dupont <link@subpop.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "lindele.h"
#include "player.h"
#include "playlist.h"
#include "mm-keys.h"

void update_now_playing(Lindele *l);
void update_buttons (Lindele *l);

void on_about_activate(GtkMenuItem *menu, Lindele *l)
{
    gtk_widget_show (GTK_WIDGET(l->about));
}

void on_button_play_clicked(GtkButton *button, Lindele *l)
{
    if(player_get_state(l->player) == STOPPED)
    {
        playlist_play (l);
        player_set_stream (l->player,playlist_get_current_filepath(l));
        player_playpause_stream (l->player);
    }
    else if(player_get_state(l->player) == PLAYING)
    {
        player_playpause_stream (l->player);
    }
    else
    {
        playlist_play (l);
        player_playpause_stream (l->player);
    }
    update_now_playing(l);
    update_buttons(l);
}

void on_button_stop_clicked (GtkButton *button, Lindele *l)
{
    player_stop_stream (l->player);
    playlist_stop (l);
    
    update_now_playing(l);
    update_buttons (l);
}

void on_button_next_clicked (GtkButton *button, Lindele *l)
{
    playlist_next (l);
    update_now_playing(l);
    update_buttons(l);
}

void on_button_prev_clicked (GtkButton *button, Lindele *l)
{
    playlist_prev (l);
    update_now_playing(l);
    update_buttons (l);
}

void mm_play_pressed (MmKeys *m, Lindele *l)
{
    on_button_play_clicked (NULL, l);
}

void mm_stop_pressed (MmKeys *m, Lindele *l)
{
    on_button_stop_clicked (NULL, l);
}

void mm_next_pressed (MmKeys *m, Lindele *l)
{
    on_button_next_clicked (NULL, l);
}

void mm_prev_pressed (MmKeys *m, Lindele *l)
{
    on_button_prev_clicked (NULL, l);
}

void on_checkbutton_repeat_toggled (GtkButton *button, Lindele *l)
{
    l->repeat = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(button));
    update_buttons(l);
}

void on_checkbutton_random_toggled (GtkButton *button, Lindele *l)
{
    l->random = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(button));
    update_buttons(l);
}

void on_save_playlist_activate (GtkWidget *menu, Lindele *l)
{
    GtkWidget *fs;
    int response;
    GSList  *files;
    char *file;
    
    fs = gtk_file_chooser_dialog_new ("Save file",
                GTK_WINDOW(glade_xml_get_widget(l->xml,"window")),
                GTK_FILE_CHOOSER_ACTION_SAVE,
                GTK_STOCK_CANCEL,GTK_RESPONSE_CANCEL,
                GTK_STOCK_SAVE,GTK_RESPONSE_ACCEPT,
                NULL);
    gtk_dialog_set_default_response (GTK_DIALOG(fs), GTK_RESPONSE_ACCEPT);
    gtk_file_chooser_set_local_only ( GTK_FILE_CHOOSER (fs), TRUE);
    gtk_file_chooser_add_shortcut_folder ( GTK_FILE_CHOOSER (fs), g_strconcat(g_get_home_dir(),"/Music/Playlists",NULL),NULL);
    
    response = gtk_dialog_run (GTK_DIALOG(fs));
    
    file = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(fs));
    if (response == GTK_RESPONSE_ACCEPT)
    {
        playlist_save_playlist(file, l);
    }
    
    gtk_widget_destroy(fs);
}

void on_remove_file_activate (GtkWidget *menu, Lindele *l)
{
    GtkTreeIter iter;
    gtk_tree_model_get_iter(GTK_TREE_MODEL(l->liststore),&iter,l->selected);
    if(DEBUG)
        g_printf("REMOVED: %s\n",playlist_get_selected_filepath(l));
    gtk_list_store_remove(GTK_LIST_STORE(l->liststore),&iter);
    l->playlist_num = playlist_get_length(l);
    update_buttons(l);
}

void on_preferences_activate (GtkWidget *menu, Lindele *l)
{
}

void on_properties_activate (GtkWidget *menu, Lindele *l)
{
    if(g_file_test(g_find_program_in_path("musicbox"),G_FILE_TEST_IS_EXECUTABLE))
    {
        g_printf("EXECUTE: %s\n",g_strconcat("musicbox \"",playlist_get_current_filepath(l),"\"",NULL));
        gnome_execute_shell(NULL,g_strconcat("musicbox \"",playlist_get_current_filepath(l),"\"",NULL));
    }
    else
    {
        g_printf("musicbox is not installed on your system\n.");
    }
}

void on_add_folder_activate (GtkMenuItem *menu, Lindele *l)
{
    GtkWidget *fs;
    int response;
    GSList  *folders;
    
    
    fs = gtk_file_chooser_dialog_new ("Select folder",
                GTK_WINDOW(glade_xml_get_widget(l->xml,"window")),
                GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
                GTK_STOCK_CANCEL,GTK_RESPONSE_CANCEL,
                GTK_STOCK_OPEN,GTK_RESPONSE_ACCEPT,
                NULL);
    gtk_dialog_set_default_response (GTK_DIALOG(fs), GTK_RESPONSE_ACCEPT);
    
    gtk_file_chooser_set_select_multiple ( GTK_FILE_CHOOSER (fs), TRUE);
    gtk_file_chooser_set_local_only ( GTK_FILE_CHOOSER (fs), TRUE);
    gtk_file_chooser_add_shortcut_folder ( GTK_FILE_CHOOSER (fs), g_strconcat(g_get_home_dir(),"/Music",NULL),NULL);
    
    response = gtk_dialog_run (GTK_DIALOG(fs));
    
    folders = gtk_file_chooser_get_filenames (GTK_FILE_CHOOSER(fs));
    if (response == GTK_RESPONSE_ACCEPT)
    {
        g_slist_foreach (folders, (GFunc) playlist_add_folder,l);
        g_slist_foreach (folders, (GFunc) g_free, NULL);
        g_slist_free (folders);
    }
    
    update_buttons(l);
    gtk_widget_destroy(fs);
}

void on_add_file_activate (GtkMenuItem *menu, Lindele *l)
{
    GtkWidget *fs;
    int response;
    GSList  *files;
    GtkFileFilter *filter;
    
    filter = gtk_file_filter_new ();
    gtk_file_filter_set_name (filter, "Audio files");
    gtk_file_filter_add_mime_type (filter, "application/ogg");
    gtk_file_filter_add_mime_type (filter, "audio/x-mp3");
    gtk_file_filter_add_mime_type (filter, "audio/x-flac");
    
    
    fs = gtk_file_chooser_dialog_new ("Select file",
                GTK_WINDOW(glade_xml_get_widget(l->xml,"window")),
                GTK_FILE_CHOOSER_ACTION_OPEN,
                GTK_STOCK_CANCEL,GTK_RESPONSE_CANCEL,
                GTK_STOCK_ADD,GTK_RESPONSE_ACCEPT,
                NULL);
    gtk_dialog_set_default_response (GTK_DIALOG(fs), GTK_RESPONSE_ACCEPT);
    
    gtk_file_chooser_set_select_multiple ( GTK_FILE_CHOOSER (fs), TRUE);
    gtk_file_chooser_set_local_only ( GTK_FILE_CHOOSER (fs), TRUE);
    gtk_file_chooser_add_filter ( GTK_FILE_CHOOSER (fs), filter);
    gtk_file_chooser_add_shortcut_folder ( GTK_FILE_CHOOSER (fs), g_strconcat(g_get_home_dir(),"/Music",NULL),NULL);
    
    response = gtk_dialog_run (GTK_DIALOG(fs));
    
    files = gtk_file_chooser_get_filenames (GTK_FILE_CHOOSER(fs));
    if (response == GTK_RESPONSE_ACCEPT)
    {
        g_slist_foreach (files, (GFunc) playlist_add_file,l);
        g_slist_foreach (files, (GFunc) g_free, NULL);
        g_slist_free (files);
    }
    
    update_buttons(l);
    gtk_widget_destroy(fs);
}

void on_load_playlist_activate (GtkWidget *menu, Lindele *l)
{
    GtkWidget *fs;
    int response;
    GSList  *files;
    char *file;
    GtkFileFilter *filter,*filter_all;
    
    filter = gtk_file_filter_new ();
    gtk_file_filter_set_name (filter, "Playlist files");
    gtk_file_filter_add_mime_type (filter, "audio/x-scpls");
    gtk_file_filter_add_mime_type (filter, "audio/x-mpegurl");
    
    filter_all = gtk_file_filter_new ();
    gtk_file_filter_set_name (filter_all, "All files");
    gtk_file_filter_add_pattern (filter_all, "*");
    
    
    fs = gtk_file_chooser_dialog_new ("Select file",
                GTK_WINDOW(glade_xml_get_widget(l->xml,"window")),
                GTK_FILE_CHOOSER_ACTION_OPEN,
                GTK_STOCK_CANCEL,GTK_RESPONSE_CANCEL,
                GTK_STOCK_OPEN,GTK_RESPONSE_ACCEPT,
                NULL);
    gtk_dialog_set_default_response (GTK_DIALOG(fs), GTK_RESPONSE_ACCEPT);
    
    gtk_file_chooser_set_select_multiple ( GTK_FILE_CHOOSER (fs), TRUE);
    gtk_file_chooser_set_local_only ( GTK_FILE_CHOOSER (fs), TRUE);
    gtk_file_chooser_add_filter ( GTK_FILE_CHOOSER (fs), filter);
    gtk_file_chooser_add_filter ( GTK_FILE_CHOOSER (fs), filter_all);
    gtk_file_chooser_add_shortcut_folder ( GTK_FILE_CHOOSER (fs), g_strconcat(g_get_home_dir(),"/Music/Playlists",NULL),NULL);
    
    response = gtk_dialog_run (GTK_DIALOG(fs));
    
    files = gtk_file_chooser_get_filenames (GTK_FILE_CHOOSER(fs));
    if (response == GTK_RESPONSE_ACCEPT)
    {
        g_slist_foreach (files, (GFunc) playlist_load_playlist,l);
        g_slist_foreach (files, (GFunc) g_free, NULL);
        g_slist_free (files);
    }
    
    update_buttons(l);
    gtk_widget_destroy(fs);
}

void on_new_playlist_activate (GtkWidget *menu, Lindele *l)
{
    gtk_list_store_clear (l->liststore);
    l->playlist_num = 0;
    
    update_buttons(l);
}

void on_jump_to_playing_activate (GtkWidget *menu, Lindele *l)
{
    GtkTreeIter iter;
    GtkTreePath *path;
    
    gtk_tree_model_get_iter(GTK_TREE_MODEL(l->liststore),&iter,l->current);
    path = gtk_tree_model_get_path(GTK_TREE_MODEL(l->liststore),&iter);
    
    gtk_tree_selection_select_path(l->selector,path);
    gtk_tree_view_set_cursor(GTK_TREE_VIEW(l->treeview),path,NULL,FALSE);
}

gboolean on_tree_popup_menu (GtkWidget *widget)
{
    g_printf("got popup-menu signal\n");
    popup_context_menu(widget,NULL);
    return TRUE;
}

void update_buttons (Lindele *l)
{
    GtkTreeIter iter;
    
    if(player_get_state(l->player) == PLAYING)
        gtk_image_set_from_file (GTK_IMAGE(glade_xml_get_widget(l->xml,"image159")),PAUSE_PNG);
    else
        gtk_image_set_from_file (GTK_IMAGE(glade_xml_get_widget(l->xml,"image159")),PLAY_PNG);
        
    if(l->playlist_num == 0)
    {
        gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(l->xml,"button_prev")),FALSE);
        gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(l->xml,"button_play")),FALSE);
        gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(l->xml,"button_stop")),FALSE);
        gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(l->xml,"button_next")),FALSE);
    }
    else if(l->random)
    {
        gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(l->xml,"button_prev")),TRUE);
        gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(l->xml,"button_play")),TRUE);
        gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(l->xml,"button_stop")),TRUE);
        gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(l->xml,"button_next")),TRUE);
    }
    else if(l->playlist_num == 1)
    {
        gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(l->xml,"button_prev")),FALSE);
        gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(l->xml,"button_play")),TRUE);
        gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(l->xml,"button_stop")),TRUE);
        gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(l->xml,"button_next")),FALSE);
    }
    else
    {
        gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(l->xml,"button_prev")),TRUE);
        gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(l->xml,"button_play")),TRUE);
        gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(l->xml,"button_stop")),TRUE);
        gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(l->xml,"button_next")),TRUE);
    
        gtk_tree_model_get_iter(GTK_TREE_MODEL(l->liststore),&iter,l->current);
        if(!gtk_tree_model_iter_next(GTK_TREE_MODEL(l->liststore),&iter))
        {
            gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(l->xml,"button_next")),FALSE);
        }
        
        gtk_tree_model_get_iter(GTK_TREE_MODEL(l->liststore),&iter,l->current);
        if(!gtk_tree_model_iter_previous(GTK_TREE_MODEL(l->liststore),&iter))
        {
            gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(l->xml,"button_prev")),FALSE);
        }
    }
}

void update_now_playing(Lindele *l)
{
    GtkTreeIter iter;
    gchar *artist;
    gchar *title;
    gchar *album;
    
    if(gtk_tree_model_get_iter(GTK_TREE_MODEL(l->liststore),&iter,l->current))
    {
        gtk_tree_model_get(GTK_TREE_MODEL(l->liststore),&iter,ARTIST_COLUMN,&artist,
                                                           TITLE_COLUMN,&title,
                                                           ALBUM_COLUMN,&album,-1);
    }
    else
    {
        gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(l->xml,"label_song")),"Failed to retrieve song data");
    }
    
    if(gst_element_get_state(GST_ELEMENT(l->player)) == GST_STATE_READY)
        gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(l->xml,"label_song")),"<i>Not playing</i>\n");
    else
        gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(l->xml,"label_song")),g_strconcat("<span size=\"large\"><b>",title,"</b></span>\n<i>",artist,"</i> from <i>",album,"</i>",NULL));
}
