/*
 *  Copyright (C) 2004 Link Dupont <link@subpop.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "lindele.h"
#include "gui.h"
#include "player.h"
#include "playlist.h"

static void lindele_class_init (LindeleClass *klass);
static void lindele_init       (Lindele      *object);
static void lindele_finalize   (GObject      *object);

static GObjectClass *parent_class;

static GType type = 0;

GType
lindele_get_type (void)
{
	if (!type) {
		static const GTypeInfo info = {
				sizeof (LindeleClass),
				NULL,           /* base_init */
				NULL,           /* base_finalize */
				(GClassInitFunc) lindele_class_init,
				NULL,           /* class_finalize */
				NULL,           /* class_data */
				sizeof (Lindele),
				0,
				(GInstanceInitFunc) lindele_init,
			};

		type = g_type_register_static (G_TYPE_OBJECT, "Lindele",
					       &info, 0);
	}

	return type;
}

static void
lindele_class_init (LindeleClass *klass)
{
	GObjectClass *object_class;

	parent_class = g_type_class_peek_parent (klass);
	object_class = (GObjectClass*) klass;

	object_class->finalize = lindele_finalize;
}

static void
lindele_finalize (GObject *object)
{
	parent_class->finalize (G_OBJECT(object));
}

static void
lindele_init (Lindele *object)
{
    GtkTreeViewColumn *column;
    GtkCellRenderer *cellrenderer;
    const gchar *name = "Lindelë";
    const gchar *version = VERSION;
    const gchar *copyright = "Copyright © 2004 Link Dupont";
    const gchar *comments = "A simple music player for GNOME";
    const gchar *authors[] = {"Link Dupont <link@subpop.net>",NULL};
    const gchar *documenters[] = {NULL};
    const gchar *translator_credits = NULL;
    GdkPixbuf *logo_pixbuf = gdk_pixbuf_new_from_file(LINDELE_PNG,NULL);
    
    object->mmkeys = mmkeys_new();
    object->player = init_player();
    object->about = gnome_about_new(name,version,copyright,comments,authors,documenters,translator_credits,logo_pixbuf);
    object->playlist_num = 0;
    object->random = FALSE;
    object->repeat = FALSE;

    gchar* gladepath = g_strconcat(DATADIR,"/lindele/lindele.glade",NULL);
	//gchar* gladepath = g_strconcat("/home/link/Hacking/lindele/lindele-head/data/lindele.glade",NULL);
	
	object->xml = glade_xml_new (gladepath, NULL, NULL);
	glade_xml_signal_autoconnect (object->xml);
	
	/* Create the listview & listmodel */
	object->liststore = gtk_list_store_new (N_COLUMNS, 
	                    GDK_TYPE_PIXBUF, G_TYPE_STRING,
						G_TYPE_STRING, G_TYPE_STRING,
						G_TYPE_STRING);
	g_assert(object->liststore);
	
	object->treeview = gtk_tree_view_new_with_model (GTK_TREE_MODEL (object->liststore));
	g_assert(object->treeview);
	
	cellrenderer = gtk_cell_renderer_pixbuf_new();
	column = gtk_tree_view_column_new_with_attributes ("", cellrenderer, "pixbuf", PLAYING_COLUMN,NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW (object->treeview), column);
	
	cellrenderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ("Artist", cellrenderer, "text", ARTIST_COLUMN, NULL);
	gtk_tree_view_column_set_reorderable(column,TRUE);
	gtk_tree_view_column_set_resizable(column,TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (object->treeview), column);
	column = gtk_tree_view_column_new_with_attributes ("Title",cellrenderer,"text",TITLE_COLUMN,NULL);
	gtk_tree_view_column_set_reorderable(column,TRUE);
	gtk_tree_view_column_set_resizable(column,TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (object->treeview),column);
	/* Disabled for now
	column = gtk_tree_view_column_new_with_attributes ("Album",cellrenderer,"text",ALBUM_COLUMN,NULL);
	gtk_tree_view_column_set_reorderable(column,TRUE);
	gtk_tree_view_column_set_resizable(column,TRUE);
	gtk_tree_view_append_column (GTK_TREE_VIEW (object->treeview),column); */
	
    object->selector = gtk_tree_view_get_selection (GTK_TREE_VIEW(object->treeview));
	gtk_tree_selection_set_mode (object->selector,GTK_SELECTION_SINGLE);
	object->current = gtk_tree_path_new_from_string("0");
	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (object->treeview), TRUE);
	gtk_tree_view_set_reorderable (GTK_TREE_VIEW (object->treeview), TRUE);
	
	gtk_widget_show(GTK_WIDGET(object->treeview));
	gtk_container_add (GTK_CONTAINER (glade_xml_get_widget(object->xml,"scrolledwindow")), GTK_WIDGET(object->treeview));
	
	g_signal_connect (G_OBJECT (object->selector), "changed",G_CALLBACK (selection_changed),object);
	g_signal_connect (G_OBJECT(object->treeview), "row-activated",G_CALLBACK(row_activated),object);
	g_signal_connect (G_OBJECT (object->player), "eos", G_CALLBACK (player_got_eos), object);
	g_signal_connect (G_OBJECT (object->treeview),"popup-menu",G_CALLBACK(on_tree_popup_menu),object);
	
	/* Connect the button callbacks. */
	g_signal_connect (G_OBJECT (glade_xml_get_widget(object->xml,"button_prev")),"clicked",G_CALLBACK(on_button_prev_clicked),object);
	g_signal_connect (G_OBJECT (glade_xml_get_widget(object->xml,"button_play")),"clicked",G_CALLBACK(on_button_play_clicked),object);
	g_signal_connect (G_OBJECT (glade_xml_get_widget(object->xml,"button_stop")),"clicked",G_CALLBACK(on_button_stop_clicked),object);
	g_signal_connect (G_OBJECT (glade_xml_get_widget(object->xml,"button_next")),"clicked",G_CALLBACK(on_button_next_clicked),object);
	g_signal_connect (G_OBJECT (glade_xml_get_widget(object->xml,"checkbutton_random")),"toggled",G_CALLBACK(on_checkbutton_random_toggled),object);
	g_signal_connect (G_OBJECT (glade_xml_get_widget(object->xml,"checkbutton_repeat")),"toggled",G_CALLBACK(on_checkbutton_repeat_toggled),object);
	
	/* Connect the menu item callbacks. */
	g_signal_connect (G_OBJECT (glade_xml_get_widget(object->xml,"m_add_file")),"activate",G_CALLBACK(on_add_file_activate),object);
	g_signal_connect (G_OBJECT (glade_xml_get_widget(object->xml,"m_add_folder")),"activate",G_CALLBACK(on_add_folder_activate),object);
	g_signal_connect (G_OBJECT (glade_xml_get_widget(object->xml,"m_remove_file")),"activate",G_CALLBACK(on_remove_file_activate),object);
	g_signal_connect (G_OBJECT (glade_xml_get_widget(object->xml,"m_new_playlist")),"activate",G_CALLBACK(on_new_playlist_activate),object);
	g_signal_connect (G_OBJECT (glade_xml_get_widget(object->xml,"m_load_playlist")),"activate",G_CALLBACK(on_load_playlist_activate),object);
	g_signal_connect (G_OBJECT (glade_xml_get_widget(object->xml,"m_save_playlist")),"activate",G_CALLBACK(on_save_playlist_activate),object);
	g_signal_connect (G_OBJECT (glade_xml_get_widget(object->xml,"m_jump_to_playing")),"activate",G_CALLBACK(on_jump_to_playing_activate),object);
	g_signal_connect (G_OBJECT (glade_xml_get_widget(object->xml,"m_properties")),"activate",G_CALLBACK(on_properties_activate),object);
	//g_signal_connect (G_OBJECT (glade_xml_get_widget(object->xml,"m_preferences")),"activate",G_CALLBACK(on_preferences_activate),object);
	g_signal_connect (G_OBJECT (glade_xml_get_widget(object->xml,"m_about")),"activate",G_CALLBACK(on_about_activate),object);
	
	/* Connect the multimedia keys callbacks. */
	g_signal_connect (G_OBJECT (object->mmkeys), "mm_playpause", G_CALLBACK(mm_play_pressed),object);
	g_signal_connect (G_OBJECT (object->mmkeys), "mm_prev", G_CALLBACK(mm_prev_pressed),object);
	g_signal_connect (G_OBJECT (object->mmkeys), "mm_next", G_CALLBACK(mm_next_pressed),object);
	g_signal_connect (G_OBJECT (object->mmkeys), "mm_stop", G_CALLBACK(mm_stop_pressed),object);
	
	/* Disable the preferences, since its not implemented. */
	gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(object->xml,"m_preferences")),FALSE);
	if(! g_file_test(g_find_program_in_path("musicbox"),G_FILE_TEST_IS_EXECUTABLE))
	    gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(object->xml,"m_properties")),FALSE);
	
	update_buttons(object);

	gtk_widget_show (GTK_WIDGET(glade_xml_get_widget(object->xml,"window")));
}

Lindele *
lindele_new (void)
{
	return LINDELE (g_object_new (TYPE_LINDELE, NULL));
}
