/*
 *  Copyright (C) 2004 Link Dupont <link@subpop.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "lindele.h"
#include "playlist.h"

extern void update_now_playing(Lindele *l);
extern void update_buttons(Lindele *l);

static gboolean
idle_iterate (GstPlay * play)
{
	gst_bin_iterate (GST_BIN (play));
	return (GST_STATE (GST_ELEMENT (play)) == GST_STATE_PLAYING);
}

void
player_playpause_stream (GstPlay *p)
{
    if(gst_element_get_state(GST_ELEMENT(p)) == GST_STATE_READY ||
       gst_element_get_state(GST_ELEMENT(p)) == GST_STATE_PAUSED)
    {
        gst_element_set_state(GST_ELEMENT(p),GST_STATE_PLAYING);
    }
    else
    {
        gst_element_set_state(GST_ELEMENT(p),GST_STATE_PAUSED);
    }
}

void
player_stop_stream (GstPlay *p)
{
    gst_element_set_state(GST_ELEMENT(p),GST_STATE_READY);
}

void
player_set_stream (GstPlay * p, const char *s)
{
	gst_play_set_location (p, s);
}

PlayerState
player_get_state (GstPlay * p)
{
    GstElementState s;
    s = gst_element_get_state(GST_ELEMENT(p));
    
    if(s == GST_STATE_READY)
        return STOPPED;
    else if(s == GST_STATE_PAUSED)
        return PAUSED;
    else if(s == GST_STATE_PLAYING)
        return PLAYING;
}

void
player_got_eos (GstElement *e, Lindele *l)
{
    playlist_next(l);
    update_now_playing(l);
    update_buttons(l);
}

GstPlay *
init_player ()
{
	GstPlay *play;
	GstElement *source, *sink;
	GError *error = NULL;

	play = gst_play_new (&error);
	if (error)
	{
		g_print ("Error: could not create play object:\n%s\n",
			 error->message);
		g_error_free (error);
		return NULL;
	}

	source = gst_element_factory_make ("gnomevfssrc", "source");

	sink = gst_gconf_get_default_audio_sink ();
	if (!GST_IS_ELEMENT (sink))
		g_error ("Could not get default audio sink from GConf");

	if (!gst_play_set_audio_sink (play, sink))
		g_warning ("Could not set audio sink");
	if (!gst_play_set_data_src (play, source))
		g_warning ("Could not set data src");

	g_idle_add ((GSourceFunc) idle_iterate, play);
	gst_element_set_state (GST_ELEMENT (play), GST_STATE_READY);

	return play;
}
