/*
 *  Copyright (C) 2004 Link Dupont <link@subpop.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "lindele.h"
#include "player.h"
#include "tag.h"
#include "util.h"
#include <stdio.h>
#include <stdlib.h>
#include <dirent.h>

extern void update_now_playing(Lindele *l);
extern void update_buttons(Lindele *l);
gboolean gtk_tree_model_iter_previous (GtkTreeModel *tree_model, GtkTreeIter *iter);
gint playlist_get_length(Lindele *l);


void
playlist_add_file (gchar * filename, Lindele *l)
{
	GtkTreeIter iter;
	FileTag t;
	GtkTreeModel *model;
	
    init_filetag(&t);
    model = GTK_TREE_MODEL(l->liststore);
    
	if(read_tag(&t,filename))
    {
        gtk_list_store_append(l->liststore,&iter);
    	gtk_list_store_set(l->liststore,&iter,
    			TITLE_COLUMN,t.title,
    			ARTIST_COLUMN,t.artist,
    			ALBUM_COLUMN,t.album,
    			FILE_COLUMN,filename,-1);
    	l->current = gtk_tree_model_get_path(GTK_TREE_MODEL(l->liststore),&iter);
    	gtk_tree_selection_get_selected(l->selector,&model,&iter);
    	l->selected = gtk_tree_model_get_path(GTK_TREE_MODEL(l->liststore),&iter);
    	l->playlist_num = playlist_get_length(l);
    	if(DEBUG)
    	   g_printf("ADDED: %s\n", filename);
    }
    else
        if(DEBUG)
            g_printf("FAIL: %s\n", filename);
}

void
playlist_add_folder (gchar * folder, Lindele *l)
{
    DIR *dip;
    struct dirent *dit;
    
    dip = opendir(folder);
    while ((dit = readdir(dip)) != NULL)
    {
        if(dit->d_name[0] == '.')
        {
            if(DEBUG)
                g_printf("Matched a file beginning with ., ignoring\n");
        }
        else
        {
            if(dit->d_type == DT_DIR)
            {
                playlist_add_folder(g_strconcat(folder,"/",dit->d_name,NULL), l);
            }
            playlist_add_file (g_strconcat(folder,"/",dit->d_name,NULL), l);
        }
    }
    closedir(dip);
}

void
playlist_load_playlist (gchar *filename, Lindele *l)
{
    if(g_str_equal((char *) gnome_vfs_get_file_mime_type (filename,NULL,TRUE),"audio/x-scpls"))
    {
        /* SHOUTCast PLS
         * Implementation heavily based on beep-media-player's playlist_load_pls function */
        guint i, count, added_count = 0;
        gchar key[10];
        gchar *line;

        g_return_val_if_fail(filename != NULL, 0);

        if (!str_has_suffix_nocase(filename, ".pls"))
            return 0;

        if (!(line = read_ini_string(filename, "playlist", "NumberOfEntries")))
            return 0;

        count = atoi(line);
        g_free(line);

        for (i = 1; i <= count; i++)
        {
            g_snprintf(key, sizeof(key), "File%d", i);
            if ((line = read_ini_string(filename, "playlist", key)))
            {
                playlist_add_file(line, l);
                g_free(line);
            }
        }
    }
    else if(g_str_equal((char *) gnome_vfs_get_file_mime_type (filename,NULL,TRUE),"audio/x-mpegurl"))
    {
        /* M3U
         * Implementation heavily based on beep-media-player's playlist_load_m3u function */
        FILE *file;
        gchar *line;
        gchar *ext_info = NULL, *ext_title = NULL;
        gsize line_len = 1024;
        gint ext_len;
        gboolean is_extm3u = FALSE;
        guint added_count = 0;

        if (!(file = fopen(filename, "r")))
            return 0;

        line = g_malloc(line_len);
        while (fgets(line, line_len, file))
        {
            while (strlen(line) == line_len - 1 && line[strlen(line) - 1] != '\n')
            {
                line_len += 1024;
                line = g_realloc(line, line_len);
                fgets(&line[strlen(line)], 1024, file);
            }

            while (line[strlen(line) - 1] == '\r' ||
                   line[strlen(line) - 1] == '\n')
                line[strlen(line) - 1] = '\0';

            if (str_has_prefix_nocase(line, "#EXTM3U"))
            {
                is_extm3u = TRUE;
                continue;
            }

            if (is_extm3u && str_has_prefix_nocase(line, "#EXTINF:"))
            {
                str_replace_in(&ext_info, g_strdup(line));
                continue;
            }

            if (line[0] == '#')
            {
                if (ext_info)
                {
                    g_free(ext_info);
                    ext_info = NULL;
                }
                continue;
            }

            if (is_extm3u)
            {
                if (ext_info)
                    parse_extm3u_info(ext_info, &ext_title, &ext_len);
                g_free(ext_info);
                ext_info = NULL;
            }

            playlist_add_file(line, l);
        }

        fclose(file);
        g_free(line);
    }
    else
    {
        /* Assuming its the plain-text list-of-paths playlist format. :) */
        char file[5000];
        gchar *temp;
        FILE *stream;
        
        stream = fopen ((char *) filename, "r");
        
        if(stream == NULL)
        {
            g_printf("Failed to open the file for reading.\n");
        }
        else
        {
            while(fgets(file,5000,stream) != NULL)
            {
                /* This little bit of magic is brought to you by:
                 * http://icculus.org/cgi-bin/viewcvs/cgi/viewcvs.cgi/smpeg2/gtv.c?rev=1.2
                 */
                temp = strchr(file, '\n');
                if(temp)
                {
                    if(*(temp - 1) == '\r')
                        *(temp - 1) = '\0';
                    *temp = '\0';
                }
                playlist_add_file(file, l);
            }
            fclose(stream);
        }
    }
}

gint
playlist_get_length(Lindele *l)
{
    GtkTreeIter iter;
    GtkTreePath *path;
    gint count=1;
    
    path = gtk_tree_path_new_from_string("0");
    gtk_tree_model_get_iter(GTK_TREE_MODEL(l->liststore),&iter,path);
    
    while(gtk_tree_model_iter_next(GTK_TREE_MODEL(l->liststore),&iter))
        count++;
    
    return count;
}

void
playlist_save_playlist (gchar * filename, Lindele *l)
{
    /* Implementation based heavily on beep-media-player's playlist_save function. */
    if(g_str_equal((char *) gnome_vfs_get_file_mime_type (filename,NULL,TRUE),"audio/x-scpls"))
    {
        FILE *file;
        GtkTreeIter iter;
        GtkTreePath *path;
        gchar *filepath;

        if ((file = fopen(filename, "w")) == NULL)
            g_printf("Failed to open file for writing\n");

        g_fprintf(file, "[playlist]\n");
        g_fprintf(file, "NumberOfEntries=%d\n", playlist_get_length(l));
        
        path = gtk_tree_path_new_from_string("0");
        gtk_tree_model_get_iter(GTK_TREE_MODEL(l->liststore),&iter,path);
        do
        {
            gtk_tree_model_get(GTK_TREE_MODEL(l->liststore),&iter,FILE_COLUMN,&filepath,-1);
            path = gtk_tree_model_get_path(GTK_TREE_MODEL(l->liststore),&iter);
            g_fprintf(file, "File%d=%s\n", atoi(gtk_tree_path_to_string(path)) + 1,filepath);
        } while(gtk_tree_model_iter_next(GTK_TREE_MODEL(l->liststore),&iter));
        
        fclose(file);
    }
    else if(g_str_equal((char *) gnome_vfs_get_file_mime_type (filename,NULL,TRUE),"audio/x-mpegurl"))
    {
        FILE *file;
        GtkTreeIter iter;
        GtkTreePath *path;
        gchar *filepath;

        if ((file = fopen(filename, "w")) == NULL)
            g_printf("Failed to open file for writing\n");
            
        g_fprintf(file, "#EXTM3U\n");
        path = gtk_tree_path_new_from_string("0");
        gtk_tree_model_get_iter(GTK_TREE_MODEL(l->liststore),&iter,path);
        do
        {
            gtk_tree_model_get(GTK_TREE_MODEL(l->liststore),&iter,FILE_COLUMN,&filepath,-1);
            path = gtk_tree_model_get_path(GTK_TREE_MODEL(l->liststore),&iter);
            g_fprintf(file, "%s\n", filepath);
        } while(gtk_tree_model_iter_next(GTK_TREE_MODEL(l->liststore),&iter));
        
        fclose(file);
    }
}

gchar*
playlist_get_current_filepath(Lindele *l)
{
	GtkTreeIter iter;
	gchar *path;

	if(!gtk_tree_model_get_iter(GTK_TREE_MODEL(l->liststore),&iter,l->current))
		return NULL;
	gtk_tree_model_get(GTK_TREE_MODEL(l->liststore),&iter,FILE_COLUMN,&path,-1);

	return path;
}

gchar*
playlist_get_selected_filepath(Lindele *l)
{
	GtkTreeIter iter;
	gchar *path;

	if(!gtk_tree_model_get_iter(GTK_TREE_MODEL(l->liststore),&iter,l->selected))
		return NULL;
	gtk_tree_model_get(GTK_TREE_MODEL(l->liststore),&iter,FILE_COLUMN,&path,-1);

	return path;
}

void
playlist_play (Lindele *l)
{
    GtkTreeIter iter;
    
    gtk_tree_model_get_iter (GTK_TREE_MODEL(l->liststore),&iter,l->selected);
    
    gtk_list_store_set(l->liststore,&iter,PLAYING_COLUMN,gdk_pixbuf_new_from_file(PLAYING_PNG,NULL),-1);
    l->current = gtk_tree_model_get_path(GTK_TREE_MODEL(l->liststore),&iter);
}

void
playlist_stop (Lindele *l)
{
    GtkTreeIter iter;
    
    gtk_tree_model_get_iter (GTK_TREE_MODEL(l->liststore),&iter,l->current);
    gtk_list_store_set(l->liststore,&iter,PLAYING_COLUMN,NULL,-1);
}

void
playlist_next (Lindele *l)
{
    GtkTreeIter iter;
    GtkTreePath *path;
    
    /* Clear the last played "playing" icon. */
    gtk_tree_model_get_iter(GTK_TREE_MODEL(l->liststore),&iter,l->current);
    gtk_list_store_set(l->liststore,&iter,PLAYING_COLUMN,NULL,-1);
    
    if(l->random)
    {
        int r;
        char s[100];
        r = g_random_int_range(0,l->playlist_num);
        g_sprintf(s,"%d",r);
        
        /* Set the current path to the randomly selected one, and start playing. */
        l->current = gtk_tree_path_new_from_string(s);
    }
    else if(gtk_tree_model_iter_next (GTK_TREE_MODEL(l->liststore),&iter))
    {
        /* Select the next track in the playlist. */
        l->current = gtk_tree_model_get_path(GTK_TREE_MODEL(l->liststore),&iter);
    }
    else if(l->repeat)
    {
        l->current = gtk_tree_path_new_from_string("0");
    }
    
    /* Set new stream & start playing. */
    player_set_stream (l->player,playlist_get_current_filepath(l));
    gst_element_set_state (GST_ELEMENT(l->player),GST_STATE_PLAYING);
    
    /* Set the new "playing" icon. */
    gtk_tree_model_get_iter (GTK_TREE_MODEL(l->liststore),&iter,l->current);
    gtk_list_store_set(l->liststore,&iter,PLAYING_COLUMN,gdk_pixbuf_new_from_file(PLAYING_PNG,NULL),-1);
    gtk_tree_model_get_iter(GTK_TREE_MODEL(l->liststore),&iter,l->current);
    
    /* Select the current song & jump to it. */
    path = gtk_tree_model_get_path(GTK_TREE_MODEL(l->liststore),&iter);
    gtk_tree_selection_select_path(l->selector,path);
    gtk_tree_view_set_cursor(GTK_TREE_VIEW(l->treeview),path,NULL,FALSE);
    
    if(DEBUG)
        g_printf("NEXT: %s\n",playlist_get_current_filepath(l));
}

void
playlist_prev (Lindele *l)
{
    GtkTreeIter iter;
    GtkTreePath *path;
    
    /* Clear the last "playing" icon. */
    gtk_tree_model_get_iter(GTK_TREE_MODEL(l->liststore),&iter,l->current);
    gtk_list_store_set(l->liststore, &iter, PLAYING_COLUMN,NULL,-1);
    
    /* Iterate back one, set new stream & start playing. */
    gtk_tree_model_iter_previous (GTK_TREE_MODEL(l->liststore),&iter);
    l->current = gtk_tree_model_get_path(GTK_TREE_MODEL(l->liststore),&iter);
    player_set_stream (l->player,playlist_get_current_filepath(l));
    gst_element_set_state(GST_ELEMENT(l->player), GST_STATE_PLAYING);
    
    /* Set new "playing" icon. */
    gtk_list_store_set(l->liststore,&iter,PLAYING_COLUMN,gdk_pixbuf_new_from_file(PLAYING_PNG,NULL),-1);
   
    /* Select the current song & jump to it. */
    //gtk_tree_model_get_iter(GTK_TREE_MODEL(liststore),&iter,current);
    path = gtk_tree_model_get_path(GTK_TREE_MODEL(l->liststore),&iter);
    gtk_tree_selection_select_path(l->selector,path);
    gtk_tree_view_set_cursor(GTK_TREE_VIEW(l->treeview),path,NULL,FALSE);
    
    if(DEBUG)
        g_printf("PREV: %s\n",playlist_get_current_filepath(l));
}

void
selection_changed(GtkTreeSelection *selection, Lindele *l)
{
	GtkTreeIter iter;
	GtkTreeModel *model;
	model = GTK_TREE_MODEL(l->liststore);

	if(gtk_tree_selection_get_selected(l->selector,&model,&iter))
	{
	    l->last = l->current;
		l->selected = gtk_tree_model_get_path(GTK_TREE_MODEL(l->liststore),&iter);
		if(DEBUG)
    	   g_printf("SELECTED: %s\n",playlist_get_selected_filepath(l));
	}
	else
	{
	   gtk_tree_model_get_iter (GTK_TREE_MODEL(l->liststore),&iter,l->selected);
	   l->selected = gtk_tree_model_get_path(GTK_TREE_MODEL(l->liststore),&iter);
	   gtk_tree_selection_select_path(l->selector,l->selected);
    }
}

void
row_activated (GtkTreeView *treeview, GtkTreePath *p, GtkTreeViewColumn *c, Lindele *l)
{
    GtkTreeIter iter;
    
    /* Clear the last "playing" icon. */
    gtk_tree_model_get_iter (GTK_TREE_MODEL(l->liststore),&iter,l->last);
    gtk_list_store_set(l->liststore,&iter,PLAYING_COLUMN,NULL,-1);
    
    /* Set to the new selected row & set "playing" icon. */
    gtk_tree_model_get_iter (GTK_TREE_MODEL(l->liststore),&iter,l->selected);
    gtk_list_store_set(l->liststore,&iter,PLAYING_COLUMN,gdk_pixbuf_new_from_file(PLAYING_PNG,NULL),-1);
    l->current = l->selected;

    /* Set new stream & start playing. */
    player_set_stream (l->player,playlist_get_current_filepath(l));
    gst_element_set_state(GST_ELEMENT(l->player), GST_STATE_PLAYING);
    
    if(DEBUG)
        g_printf("ACTIVATED: %s\n",playlist_get_current_filepath(l));
        
    update_now_playing(l);
    update_buttons(l);
}

void
popup_context_menu(GtkWidget *widget, GdkEventButton *event)
{
    g_printf("entry into popup_context_menu\n");
    GtkWidget *menu,*menuitem;
    int button, event_time;
    
    menu = gtk_menu_new ();
    g_signal_connect (menu,"deactivate",G_CALLBACK(gtk_widget_destroy),NULL);
    
    /* Add menu items */
    menuitem = gtk_menu_item_new_with_label("Hallo!");
    gtk_menu_shell_append(GTK_MENU_SHELL(menu),menuitem);
    
    if(event)
    {
        button = event->button;
        event_time = event->time;
    }
    else
    {
        button = 0;
        event_time = gtk_get_current_event_time ();
    }
    
    gtk_menu_popup (GTK_MENU(menu),NULL,NULL,NULL,NULL,button,event_time);
}

/* Thanks to totem and its gtk-playlist.c file for this function. */
gboolean
gtk_tree_model_iter_previous (GtkTreeModel *tree_model, GtkTreeIter *iter)
{
	GtkTreePath *path;
	gboolean ret;

	path = gtk_tree_model_get_path (tree_model, iter);
	ret = gtk_tree_path_prev (path);
	if (ret == TRUE)
		gtk_tree_model_get_iter (tree_model, iter, path);

	gtk_tree_path_free (path);
	return ret;
}
